from .base import WikipediaBaseFilter


class WikipediaFilmFilter(WikipediaBaseFilter):
    keywords = [
        "film",
        "films",
        "movie",
        "movies",
        "theatrical release",
        "theatrical releases"
        "home video",
        "home videos",
    ]
